from typing import List

import aicmder as cmder
from aicmder.commands import register
from aicmder.commands.utils import _command_prefix as cmd
import argparse
import os
from shutil import copyfile
from pprint import pprint
@register(name='{}.init'.format(cmd), description='init.')
class InitCommand:
    
    _export_helper = 'export_helper.py'
    def execute(self, argv: List) -> bool:
        cur_path = os.getcwd()
        model_dir = os.path.join(cur_path, 'model')
        
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)
            os.system('touch {}'.format(os.path.join(model_dir, '__init__.py')))
        
        dest = os.path.join(model_dir, self._export_helper)
        if not os.path.exists(dest):
            src = os.path.abspath(os.path.dirname(__file__))
            src = os.path.join(src, '../../model', self._export_helper)
            copyfile(src, dest)
        


#coding:utf-8
from typing import Any
from collections import defaultdict


_command_prefix = 'aicmder'

def _CommandDict():
    return defaultdict(_CommandDict)


_commands = _CommandDict()


def register(name: str, description: str = '') -> Any:
    '''
    Register a subcommand in the command list

    Args:
        name(str) : The name of the command, separated by '.' (e.g, aicmder.serving)
        description(str) : The description of the specified command showd in the help command, if not description given, this command would not be shown in help command. Default is None.
    '''

    def _warpper(command):
        items = name.split('.')

        com = _commands
        for item in items:
            com = com[item]
        com['_entry'] = command
        if description:
            com['_description'] = description
        return command

    return _warpper


def get_command(name: str) -> Any:
    items = name.split('.')
    com = _commands
    for item in items:
        com = com[item]

    return com['_entry']


def execute():
    import sys
    com = _commands
    for index, _argv in enumerate([_command_prefix] + sys.argv[1:]):
        if _argv not in com:
            break
        com = com[_argv]
    else:
        index += 1

    return com['_entry']().execute(sys.argv[index:])
